(function() {
    'use strict';

    angular.module('JahiaOAuthApp').controller('JExperienceOAuthDataMapperController', JExperienceOAuthController);

    JExperienceOAuthController.$inject = ['$routeParams', '$http', 'settingsService', 'helperService', 'i18nService', 'jahiaContext'];

    function JExperienceOAuthController($routeParams, $http, settingsService, helperService, i18nService, jahiaContext) {
        var vm = this;

        // Variables
        vm.isActivate = false;
        vm.connectorProperties = [];
        vm.mapperProperties = [];
        vm.mapping = [];
        vm.selectedPropertyFromConnector = '';
        vm.expandedCard = false;
        vm.issueWithJExperienceSettings = true;

        // Functions
        vm.saveMapperSettings = saveMapperSettings;
        vm.addMapping = addMapping;
        vm.removeMapping = removeMapping;
        vm.getConnectorI18n = getConnectorI18n;
        vm.toggleCard = toggleCard;
        vm.orderByConnector = orderByConnector;

        _init();

        function saveMapperSettings() {
            var isMappingComplete = true;
            angular.forEach(vm.mapping, function(mapped) {
                if (!mapped.mapper || !mapped.connector) {
                    isMappingComplete =false;
                }
            });
            if (!isMappingComplete) {
                helperService.errorToast(i18nService.message('joant_marketingFactoryOAuthView.message.error.incompleteMapping'));
                return false;
            }

            var mandatoryPropertyAreMapped = true;
            angular.forEach(vm.mapperProperties, function(property) {
                if (property.mandatory) {
                    if (_isNotMapped(property.name, 'mapper')) {
                        mandatoryPropertyAreMapped = false
                    }
                }
            });
            if (vm.isActivate && !mandatoryPropertyAreMapped) {
                helperService.errorToast(i18nService.message('joant_marketingFactoryOAuthView.message.error.mandatoryPropertiesNotMapped'));
                return false;
            }

            settingsService.setMapperMapping({
                connectorServiceName: $routeParams.connectorServiceName,
                mapperServiceName: 'jExperienceOAuthDataMapper',
                nodeType: 'joant:marketingFactoryOAuthSettings',
                isActivate: vm.isActivate,
                mapping: vm.mapping
            }).success(function() {
                helperService.successToast(i18nService.message('joant_marketingFactoryOAuthView.message.success.mappingSaved'));
            }).error(function(data) {
                helperService.errorToast(i18nService.message('joant_marketingFactoryOAuthView.message.label') + ' ' + data.error);
            });
        }

        function addMapping() {
            if (vm.selectedPropertyFromConnector) {
                vm.mapping.push({
                    connector: vm.selectedPropertyFromConnector
                });
                vm.selectedPropertyFromConnector = '';
            }
        }

        function removeMapping(index) {
            vm.mapping.splice(index, 1);
        }

        function getConnectorI18n(value) {
            return i18nService.message($routeParams.connectorServiceName + '.label.' + value);
        }

        function _getMapperI18n(value) {
            return i18nService.message('jExperienceOAuthDataMapper.label.' + value);
        }

        function toggleCard() {
            vm.expandedCard = !vm.expandedCard;
        }

        function orderByConnector(property) {
            return getConnectorI18n(property.name);
        }

        // Private functions under this line
        function _init() {
            i18nService.addKey(jxpoai18n);

            settingsService.getMapperMapping({
                connectorServiceName: $routeParams.connectorServiceName,
                mapperServiceName: 'jExperienceOAuthDataMapper'
            }).success(function (data) {
                if (!angular.equals(data, {})) {
                    vm.isActivate = data.isActivate;
                    vm.mapping = data.mapping;
                }
            }).error(function (data) {
                helperService.errorToast(i18nService.message('joant_marketingFactoryOAuthView.message.label') + ' ' + data.error);
            });

            settingsService.getConnectorProperties({
                connectorServiceName: $routeParams.connectorServiceName
            }).success(function(data) {
                vm.connectorProperties = data.connectorProperties;
            }).error(function (data) {
                helperService.errorToast(i18nService.message('joant_marketingFactoryOAuthView.message.label') + ' ' + data.error);
            });

            $http.get(jahiaContext.context + '/modules/jexperience/proxy/' + jahiaContext.siteKey + '/cxs/profiles/properties/targets/profiles')
                .then(function(data) {
                    if (data.data.length > 0) {
                        vm.issueWithJExperienceSettings = false;
                    }
                    for (var i in data.data) {
                        var tags = data.data[i].metadata.systemTags;
                        if (tags.indexOf("systemProfileProperties") == -1 && tags.indexOf("hiddenFromFormMappingProperties") == -1) {
                            var itemId = data.data[i].itemId;
                            var displayableName = !_getMapperI18n(itemId).startsWith('???') ? _getMapperI18n(itemId) : data.data[i].metadata.name;
                            vm.mapperProperties.push({
                                name: itemId,
                                displayableName: displayableName,
                                mandatory: false,
                                valueType: data.data[i].type
                            });
                        }
                    }
                }, function(data) {
                    console.error(data);
                    helperService.errorToast(i18nService.message('joant_marketingFactoryOAuthView.message.error.whenRetrievingMapperProperties'));
                });
        }

        function _isNotMapped(field, key) {
            var isNotMapped = true;
            angular.forEach(vm.mapping, function(entry) {
                if (entry[key] && entry[key].name == field) {
                    isNotMapped = false;
                }
            });
            return isNotMapped;
        }
    }
})();
